@@ -66,7 +66,7 @@ static int fts_backend_xapian_init(struct fts_backend *_backend, const char **er
 	struct xapian_fts_backend *backend =
 		(struct xapian_fts_backend *)_backend;
 	const char *const *tmp, *env;
-	unsigned int len;
+	long len;
 
 	backend->dbw = NULL;
 	backend->dbr = NULL;
@@ -76,6 +76,8 @@ static int fts_backend_xapian_init(struct fts_backend *_backend, const char **er
 	backend->oldbox = NULL;
 	backend->attachments = false;
 	verbose = 0;
+	backend->partial = 0;
+	backend->full = 0;
 
 	env = mail_user_plugin_getenv(_backend->ns->user, "fts_xapian");
 	if (env == NULL) 
@@ -86,21 +88,24 @@ static int fts_backend_xapian_init(struct fts_backend *_backend, const char **er
 
 	for (tmp = t_strsplit_spaces(env, " "); *tmp != NULL; tmp++)
 	{
-        	if (str_begins(*tmp, "partial=") && (str_to_uint(*tmp + 8, &len)>=0))
+        	if (strncmp(*tmp, "partial=",8)==0)
 		{
-			backend->partial=len;
+			len=atol(*tmp + 8);
+			if(len>0) backend->partial=len;
 		}
-        	else if (str_begins(*tmp, "full=") && (str_to_uint(*tmp + 5, &len)>=0))
+        	else if (strncmp(*tmp,"full=",5)==0)
 		{
-			backend->full=len;
+			len=atol(*tmp + 5);
+			if(len>0) backend->full=len;
 		}
-		else if (str_begins(*tmp, "verbose=") && (str_to_uint(*tmp + 8, &len)>=0))
+		else if (strncmp(*tmp,"verbose=",8)==0)
 		{
+			len=atol(*tmp + 8);
 			if(len>0) verbose=len;
 		}
-		else if (str_begins(*tmp, "attachments=") && (str_to_uint(*tmp + 12, &len)>=0))
+		else if (strncmp(*tmp,"attachments=",12)==0)
 		{
-			if(len>0) backend->attachments=true;
+			if(atol(*tmp + 12)>0) backend->attachments=true;
 		}
 		else 
 		{
@@ -152,13 +157,14 @@ static void fts_backend_xapian_deinit(struct fts_backend *_backend)
 	struct xapian_fts_backend *backend =
 		(struct xapian_fts_backend *)_backend;
 
+	if(verbose>0) i_info("Deinit %s",backend->path);
+
 	fts_backend_xapian_unset_box(backend);
 
 	if(backend->oldbox != NULL) i_free(backend->oldbox);
 	backend->oldbox = NULL;
 	if(backend->path != NULL) i_free(backend->path);
 	backend->path = NULL;
-
 	i_free(backend);
 }
 
@@ -193,7 +199,7 @@ static int fts_backend_xapian_get_last_uid(struct fts_backend *_backend,
 		i_error("FTS Xapian: %s",e.get_msg().c_str());
 		return -1;
 	}
-	if(verbose>1) i_info("FTS Xapian: Get last UID of %s = %d",backend->box->name,*last_uid_r);
+	if(verbose>0) i_info("FTS Xapian: Get last UID of %s = %d",backend->box->name,*last_uid_r);
         return 0;
 }
 
@@ -211,10 +217,13 @@ static int fts_backend_xapian_update_deinit(struct fts_backend_update_context *_
 {
 	struct xapian_fts_backend_update_context *ctx =
 		(struct xapian_fts_backend_update_context *)_ctx;
+	struct xapian_fts_backend *backend =
+		(struct xapian_fts_backend *)ctx->ctx.backend;
 
-	ctx->ctx.backend = NULL;
+	if(verbose>0) i_info("Update deinit %s",backend->path);
 
 	i_free(ctx);
+
 	return 0;
 }
 
@@ -241,15 +250,20 @@ static void fts_backend_xapian_update_expunge(struct fts_backend_update_context
 		return ;
 	}
 
+	if(verbose>0) i_info("FTS Xapian: Expunge UID=%d from %s",uid,backend->box->name);
+
     	try
 	{
 		const char * s = t_strdup_printf("Q%d",uid);
         	backend->dbw->delete_document(s);
+//		backend->dbw->commit();
 	}
 	catch(Xapian::Error e)
 	{
 		i_error("FTS Xapian: %s",e.get_msg().c_str());
 	}
+	
+	if(verbose>0) i_info("FTS Xapian: Expunge UID=%d from %s done",uid,backend->box->name);
 }
 
 static bool fts_backend_xapian_update_set_build_key(struct fts_backend_update_context *_ctx, const struct fts_backend_build_key *key)
@@ -305,7 +319,7 @@ static bool fts_backend_xapian_update_set_build_key(struct fts_backend_update_co
 	}
 
 	// Verify content-disposition
-	if((disposition != NULL) && (!backend->attachments) && (strstr(disposition,"filename=")!=NULL))
+	if((disposition != NULL) && (!backend->attachments) && ((strstr(disposition,"filename=")!=NULL) || (strstr(disposition,"attachment")!=NULL)))
 	{
 		if(verbose>0) i_info("FTS Xapian: Skipping part of type '%s' and disposition '%s'",type,disposition);
 		return FALSE;
@@ -342,6 +356,8 @@ static bool fts_backend_xapian_update_set_build_key(struct fts_backend_update_co
 		return FALSE; 
 	}
 
+//	if(verbose>0) { i_info("FTS Xapian: Indexing FIELD=%s TYPE=%s DESC=%s",field,type,disposition); }
+
 	switch (key->type)
 	{
 		case FTS_BACKEND_BUILD_KEY_HDR:
@@ -406,7 +422,9 @@ static int fts_backend_xapian_update_build_more(struct fts_backend_update_contex
 	if(ctx->tbi_uid<1) return 0;
 
 	if(data == NULL) return 0;
-	if(size<1) return 0;
+	icu::StringPiece sp_d((const char *)data,(int32_t )size);
+	icu::UnicodeString d2 = icu::UnicodeString::fromUTF8(sp_d);
+	if(d2.length() < backend->partial) return 0;
 
 	if((backend->oldbox == NULL) || (strcmp(backend->oldbox,backend->box->name)!=0))
 	{
@@ -421,23 +439,17 @@ static int fts_backend_xapian_update_build_more(struct fts_backend_update_contex
 		return -1;
 	}
 
-	char * s = (char*)i_malloc(sizeof(char)*(size+1));
-	strncpy(s,(char *)data,size);
-	s[size]=0;
-
 	bool ok=true;
 
     	if(ctx->tbi_isfield)
     	{
-        	ok=fts_backend_xapian_index_hdr(backend->dbw,ctx->tbi_uid,ctx->tbi_field, s, backend->partial,backend->full);
+        	ok=fts_backend_xapian_index_hdr(backend->dbw,ctx->tbi_uid,ctx->tbi_field, &d2, backend->partial,backend->full);
     	}
     	else
     	{
-		ok=fts_backend_xapian_index_text(backend->dbw,ctx->tbi_uid,ctx->tbi_field, s, backend->partial,backend->full);
+		ok=fts_backend_xapian_index_text(backend->dbw,ctx->tbi_uid,ctx->tbi_field, &d2, backend->partial,backend->full);
     	}
 
-	i_free(s);
-
 	backend->nb_updates++;
 	if(backend->nb_updates>XAPIAN_COMMIT_LIMIT)
 	{
